# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['AccessControlRuleSetArgs', 'AccessControlRuleSet']

@pulumi.input_type
class AccessControlRuleSetArgs:
    def __init__(__self__, *,
                 grant_rules: Optional[pulumi.Input[Sequence[pulumi.Input['AccessControlRuleSetGrantRuleArgs']]]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None):
        """
        The set of arguments for constructing a AccessControlRuleSet resource.
        :param pulumi.Input[Sequence[pulumi.Input['AccessControlRuleSetGrantRuleArgs']]] grant_rules: The access control rules to be granted by this rule set, consisting of a set of principals and roles to be granted to them.
               
               !> Name uniquely identifies a rule set resource. Ensure all the grant_rules blocks for a rule set name are present in one `AccessControlRuleSet` resource block. Otherwise, after applying changes, users might lose their role assignment even if that was not intended.
        :param pulumi.Input[_builtins.str] name: Unique identifier of a rule set. The name determines the resource to which the rule set applies. **Changing the name recreates the resource!**. Currently, only default rule sets are supported. The following rule set formats are supported:
               * `accounts/{account_id}/ruleSets/default` - account-level access control.
               * `accounts/{account_id}/servicePrincipals/{service_principal_application_id}/ruleSets/default` - access control for a specific service principal.
               * `accounts/{account_id}/groups/{group_id}/ruleSets/default` - access control for a specific group.
               * `accounts/{account_id}/budgetPolicies/{budget_policy_id}/ruleSets/default` - access control for a specific budget policy.
        """
        if grant_rules is not None:
            pulumi.set(__self__, "grant_rules", grant_rules)
        if name is not None:
            pulumi.set(__self__, "name", name)

    @_builtins.property
    @pulumi.getter(name="grantRules")
    def grant_rules(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['AccessControlRuleSetGrantRuleArgs']]]]:
        """
        The access control rules to be granted by this rule set, consisting of a set of principals and roles to be granted to them.

        !> Name uniquely identifies a rule set resource. Ensure all the grant_rules blocks for a rule set name are present in one `AccessControlRuleSet` resource block. Otherwise, after applying changes, users might lose their role assignment even if that was not intended.
        """
        return pulumi.get(self, "grant_rules")

    @grant_rules.setter
    def grant_rules(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['AccessControlRuleSetGrantRuleArgs']]]]):
        pulumi.set(self, "grant_rules", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Unique identifier of a rule set. The name determines the resource to which the rule set applies. **Changing the name recreates the resource!**. Currently, only default rule sets are supported. The following rule set formats are supported:
        * `accounts/{account_id}/ruleSets/default` - account-level access control.
        * `accounts/{account_id}/servicePrincipals/{service_principal_application_id}/ruleSets/default` - access control for a specific service principal.
        * `accounts/{account_id}/groups/{group_id}/ruleSets/default` - access control for a specific group.
        * `accounts/{account_id}/budgetPolicies/{budget_policy_id}/ruleSets/default` - access control for a specific budget policy.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)


@pulumi.input_type
class _AccessControlRuleSetState:
    def __init__(__self__, *,
                 etag: Optional[pulumi.Input[_builtins.str]] = None,
                 grant_rules: Optional[pulumi.Input[Sequence[pulumi.Input['AccessControlRuleSetGrantRuleArgs']]]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering AccessControlRuleSet resources.
        :param pulumi.Input[Sequence[pulumi.Input['AccessControlRuleSetGrantRuleArgs']]] grant_rules: The access control rules to be granted by this rule set, consisting of a set of principals and roles to be granted to them.
               
               !> Name uniquely identifies a rule set resource. Ensure all the grant_rules blocks for a rule set name are present in one `AccessControlRuleSet` resource block. Otherwise, after applying changes, users might lose their role assignment even if that was not intended.
        :param pulumi.Input[_builtins.str] name: Unique identifier of a rule set. The name determines the resource to which the rule set applies. **Changing the name recreates the resource!**. Currently, only default rule sets are supported. The following rule set formats are supported:
               * `accounts/{account_id}/ruleSets/default` - account-level access control.
               * `accounts/{account_id}/servicePrincipals/{service_principal_application_id}/ruleSets/default` - access control for a specific service principal.
               * `accounts/{account_id}/groups/{group_id}/ruleSets/default` - access control for a specific group.
               * `accounts/{account_id}/budgetPolicies/{budget_policy_id}/ruleSets/default` - access control for a specific budget policy.
        """
        if etag is not None:
            pulumi.set(__self__, "etag", etag)
        if grant_rules is not None:
            pulumi.set(__self__, "grant_rules", grant_rules)
        if name is not None:
            pulumi.set(__self__, "name", name)

    @_builtins.property
    @pulumi.getter
    def etag(self) -> Optional[pulumi.Input[_builtins.str]]:
        return pulumi.get(self, "etag")

    @etag.setter
    def etag(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "etag", value)

    @_builtins.property
    @pulumi.getter(name="grantRules")
    def grant_rules(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['AccessControlRuleSetGrantRuleArgs']]]]:
        """
        The access control rules to be granted by this rule set, consisting of a set of principals and roles to be granted to them.

        !> Name uniquely identifies a rule set resource. Ensure all the grant_rules blocks for a rule set name are present in one `AccessControlRuleSet` resource block. Otherwise, after applying changes, users might lose their role assignment even if that was not intended.
        """
        return pulumi.get(self, "grant_rules")

    @grant_rules.setter
    def grant_rules(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['AccessControlRuleSetGrantRuleArgs']]]]):
        pulumi.set(self, "grant_rules", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Unique identifier of a rule set. The name determines the resource to which the rule set applies. **Changing the name recreates the resource!**. Currently, only default rule sets are supported. The following rule set formats are supported:
        * `accounts/{account_id}/ruleSets/default` - account-level access control.
        * `accounts/{account_id}/servicePrincipals/{service_principal_application_id}/ruleSets/default` - access control for a specific service principal.
        * `accounts/{account_id}/groups/{group_id}/ruleSets/default` - access control for a specific group.
        * `accounts/{account_id}/budgetPolicies/{budget_policy_id}/ruleSets/default` - access control for a specific budget policy.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)


@pulumi.type_token("databricks:index/accessControlRuleSet:AccessControlRuleSet")
class AccessControlRuleSet(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 grant_rules: Optional[pulumi.Input[Sequence[pulumi.Input[Union['AccessControlRuleSetGrantRuleArgs', 'AccessControlRuleSetGrantRuleArgsDict']]]]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        This resource allows you to manage access rules on Databricks account level resources. For convenience we allow accessing this resource through the Databricks account and workspace.

        > This resource can be used with an account or workspace-level provider.

        > Currently, we only support managing access rules on specific object resources (service principal, group, budget policies and account) through `AccessControlRuleSet`.

        !> `AccessControlRuleSet` cannot be used to manage access rules for resources supported by databricks_permissions. Refer to its documentation for more information.

        > This resource is _authoritative_ for permissions on objects. Configuring this resource for an object will **OVERWRITE** any existing permissions of the same type unless imported, and changes made outside of Pulumi will be reset.

        ## Service principal rule set usage

        Through a Databricks workspace:

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group
        ds = databricks.get_group(display_name="Data Science")
        automation_sp = databricks.ServicePrincipal("automation_sp", display_name="SP_FOR_AUTOMATION")
        automation_sp_rule_set = databricks.AccessControlRuleSet("automation_sp_rule_set",
            name=automation_sp.application_id.apply(lambda application_id: f"accounts/{account_id}/servicePrincipals/{application_id}/ruleSets/default"),
            grant_rules=[{
                "principals": [ds.acl_principal_id],
                "role": "roles/servicePrincipal.user",
            }])
        ```

        Through AWS Databricks account:

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group creation
        ds = databricks.Group("ds", display_name="Data Science")
        automation_sp = databricks.ServicePrincipal("automation_sp", display_name="SP_FOR_AUTOMATION")
        automation_sp_rule_set = databricks.AccessControlRuleSet("automation_sp_rule_set",
            name=automation_sp.application_id.apply(lambda application_id: f"accounts/{account_id}/servicePrincipals/{application_id}/ruleSets/default"),
            grant_rules=[{
                "principals": [ds.acl_principal_id],
                "role": "roles/servicePrincipal.user",
            }])
        ```

        Through Azure Databricks account:

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group creation
        ds = databricks.Group("ds", display_name="Data Science")
        automation_sp = databricks.ServicePrincipal("automation_sp",
            application_id="00000000-0000-0000-0000-000000000000",
            display_name="SP_FOR_AUTOMATION")
        automation_sp_rule_set = databricks.AccessControlRuleSet("automation_sp_rule_set",
            name=automation_sp.application_id.apply(lambda application_id: f"accounts/{account_id}/servicePrincipals/{application_id}/ruleSets/default"),
            grant_rules=[{
                "principals": [ds.acl_principal_id],
                "role": "roles/servicePrincipal.user",
            }])
        ```

        Through GCP Databricks account:

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group creation
        ds = databricks.Group("ds", display_name="Data Science")
        automation_sp = databricks.ServicePrincipal("automation_sp", display_name="SP_FOR_AUTOMATION")
        automation_sp_rule_set = databricks.AccessControlRuleSet("automation_sp_rule_set",
            name=automation_sp.application_id.apply(lambda application_id: f"accounts/{account_id}/servicePrincipals/{application_id}/ruleSets/default"),
            grant_rules=[{
                "principals": [ds.acl_principal_id],
                "role": "roles/servicePrincipal.user",
            }])
        ```

        ## Group rule set usage

        Refer to the appropriate provider configuration as shown in the examples for service principal rule set.

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group
        ds = databricks.get_group(display_name="Data Science")
        john = databricks.get_user(user_name="john.doe@example.com")
        ds_group_rule_set = databricks.AccessControlRuleSet("ds_group_rule_set",
            name=f"accounts/{account_id}/groups/{ds_databricks_group['id']}/ruleSets/default",
            grant_rules=[{
                "principals": [john.acl_principal_id],
                "role": "roles/group.manager",
            }])
        ```

        ## Account rule set usage

        Refer to the appropriate provider configuration as shown in the examples for service principal rule set.

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group
        ds = databricks.get_group(display_name="Data Science")
        # account level group
        marketplace_admins = databricks.get_group(display_name="Marketplace Admins")
        john = databricks.get_user(user_name="john.doe@example.com")
        account_rule_set = databricks.AccessControlRuleSet("account_rule_set",
            name=f"accounts/{account_id}/ruleSets/default",
            grant_rules=[
                {
                    "principals": [john.acl_principal_id],
                    "role": "roles/group.manager",
                },
                {
                    "principals": [ds.acl_principal_id],
                    "role": "roles/servicePrincipal.manager",
                },
                {
                    "principals": [marketplace_admins.acl_principal_id],
                    "role": "roles/marketplace.admin",
                },
            ])
        ```

        ## Budget policy usage

        Access to budget policies could be controlled with this resource:

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group
        ds = databricks.get_group(display_name="Data Science")
        john = databricks.get_user(user_name="john.doe@example.com")
        this = databricks.BudgetPolicy("this",
            policy_name="data-science-budget-policy",
            custom_tags=[{
                "key": "mykey",
                "value": "myvalue",
            }])
        budget_policy_usage = databricks.AccessControlRuleSet("budget_policy_usage",
            name=this.policy_id.apply(lambda policy_id: f"accounts/{account_id}/budgetPolicies/{policy_id}/ruleSets/default"),
            grant_rules=[
                {
                    "principals": [john.acl_principal_id],
                    "role": "roles/budgetPolicy.manager",
                },
                {
                    "principals": [ds.acl_principal_id],
                    "role": "roles/budgetPolicy.user",
                },
            ])
        ```

        ## Related Resources

        The following resources are often used in the same context:

        * Group
        * User
        * ServicePrincipal

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['AccessControlRuleSetGrantRuleArgs', 'AccessControlRuleSetGrantRuleArgsDict']]]] grant_rules: The access control rules to be granted by this rule set, consisting of a set of principals and roles to be granted to them.
               
               !> Name uniquely identifies a rule set resource. Ensure all the grant_rules blocks for a rule set name are present in one `AccessControlRuleSet` resource block. Otherwise, after applying changes, users might lose their role assignment even if that was not intended.
        :param pulumi.Input[_builtins.str] name: Unique identifier of a rule set. The name determines the resource to which the rule set applies. **Changing the name recreates the resource!**. Currently, only default rule sets are supported. The following rule set formats are supported:
               * `accounts/{account_id}/ruleSets/default` - account-level access control.
               * `accounts/{account_id}/servicePrincipals/{service_principal_application_id}/ruleSets/default` - access control for a specific service principal.
               * `accounts/{account_id}/groups/{group_id}/ruleSets/default` - access control for a specific group.
               * `accounts/{account_id}/budgetPolicies/{budget_policy_id}/ruleSets/default` - access control for a specific budget policy.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[AccessControlRuleSetArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        This resource allows you to manage access rules on Databricks account level resources. For convenience we allow accessing this resource through the Databricks account and workspace.

        > This resource can be used with an account or workspace-level provider.

        > Currently, we only support managing access rules on specific object resources (service principal, group, budget policies and account) through `AccessControlRuleSet`.

        !> `AccessControlRuleSet` cannot be used to manage access rules for resources supported by databricks_permissions. Refer to its documentation for more information.

        > This resource is _authoritative_ for permissions on objects. Configuring this resource for an object will **OVERWRITE** any existing permissions of the same type unless imported, and changes made outside of Pulumi will be reset.

        ## Service principal rule set usage

        Through a Databricks workspace:

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group
        ds = databricks.get_group(display_name="Data Science")
        automation_sp = databricks.ServicePrincipal("automation_sp", display_name="SP_FOR_AUTOMATION")
        automation_sp_rule_set = databricks.AccessControlRuleSet("automation_sp_rule_set",
            name=automation_sp.application_id.apply(lambda application_id: f"accounts/{account_id}/servicePrincipals/{application_id}/ruleSets/default"),
            grant_rules=[{
                "principals": [ds.acl_principal_id],
                "role": "roles/servicePrincipal.user",
            }])
        ```

        Through AWS Databricks account:

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group creation
        ds = databricks.Group("ds", display_name="Data Science")
        automation_sp = databricks.ServicePrincipal("automation_sp", display_name="SP_FOR_AUTOMATION")
        automation_sp_rule_set = databricks.AccessControlRuleSet("automation_sp_rule_set",
            name=automation_sp.application_id.apply(lambda application_id: f"accounts/{account_id}/servicePrincipals/{application_id}/ruleSets/default"),
            grant_rules=[{
                "principals": [ds.acl_principal_id],
                "role": "roles/servicePrincipal.user",
            }])
        ```

        Through Azure Databricks account:

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group creation
        ds = databricks.Group("ds", display_name="Data Science")
        automation_sp = databricks.ServicePrincipal("automation_sp",
            application_id="00000000-0000-0000-0000-000000000000",
            display_name="SP_FOR_AUTOMATION")
        automation_sp_rule_set = databricks.AccessControlRuleSet("automation_sp_rule_set",
            name=automation_sp.application_id.apply(lambda application_id: f"accounts/{account_id}/servicePrincipals/{application_id}/ruleSets/default"),
            grant_rules=[{
                "principals": [ds.acl_principal_id],
                "role": "roles/servicePrincipal.user",
            }])
        ```

        Through GCP Databricks account:

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group creation
        ds = databricks.Group("ds", display_name="Data Science")
        automation_sp = databricks.ServicePrincipal("automation_sp", display_name="SP_FOR_AUTOMATION")
        automation_sp_rule_set = databricks.AccessControlRuleSet("automation_sp_rule_set",
            name=automation_sp.application_id.apply(lambda application_id: f"accounts/{account_id}/servicePrincipals/{application_id}/ruleSets/default"),
            grant_rules=[{
                "principals": [ds.acl_principal_id],
                "role": "roles/servicePrincipal.user",
            }])
        ```

        ## Group rule set usage

        Refer to the appropriate provider configuration as shown in the examples for service principal rule set.

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group
        ds = databricks.get_group(display_name="Data Science")
        john = databricks.get_user(user_name="john.doe@example.com")
        ds_group_rule_set = databricks.AccessControlRuleSet("ds_group_rule_set",
            name=f"accounts/{account_id}/groups/{ds_databricks_group['id']}/ruleSets/default",
            grant_rules=[{
                "principals": [john.acl_principal_id],
                "role": "roles/group.manager",
            }])
        ```

        ## Account rule set usage

        Refer to the appropriate provider configuration as shown in the examples for service principal rule set.

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group
        ds = databricks.get_group(display_name="Data Science")
        # account level group
        marketplace_admins = databricks.get_group(display_name="Marketplace Admins")
        john = databricks.get_user(user_name="john.doe@example.com")
        account_rule_set = databricks.AccessControlRuleSet("account_rule_set",
            name=f"accounts/{account_id}/ruleSets/default",
            grant_rules=[
                {
                    "principals": [john.acl_principal_id],
                    "role": "roles/group.manager",
                },
                {
                    "principals": [ds.acl_principal_id],
                    "role": "roles/servicePrincipal.manager",
                },
                {
                    "principals": [marketplace_admins.acl_principal_id],
                    "role": "roles/marketplace.admin",
                },
            ])
        ```

        ## Budget policy usage

        Access to budget policies could be controlled with this resource:

        ```python
        import pulumi
        import pulumi_databricks as databricks

        account_id = "00000000-0000-0000-0000-000000000000"
        # account level group
        ds = databricks.get_group(display_name="Data Science")
        john = databricks.get_user(user_name="john.doe@example.com")
        this = databricks.BudgetPolicy("this",
            policy_name="data-science-budget-policy",
            custom_tags=[{
                "key": "mykey",
                "value": "myvalue",
            }])
        budget_policy_usage = databricks.AccessControlRuleSet("budget_policy_usage",
            name=this.policy_id.apply(lambda policy_id: f"accounts/{account_id}/budgetPolicies/{policy_id}/ruleSets/default"),
            grant_rules=[
                {
                    "principals": [john.acl_principal_id],
                    "role": "roles/budgetPolicy.manager",
                },
                {
                    "principals": [ds.acl_principal_id],
                    "role": "roles/budgetPolicy.user",
                },
            ])
        ```

        ## Related Resources

        The following resources are often used in the same context:

        * Group
        * User
        * ServicePrincipal

        :param str resource_name: The name of the resource.
        :param AccessControlRuleSetArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(AccessControlRuleSetArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 grant_rules: Optional[pulumi.Input[Sequence[pulumi.Input[Union['AccessControlRuleSetGrantRuleArgs', 'AccessControlRuleSetGrantRuleArgsDict']]]]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = AccessControlRuleSetArgs.__new__(AccessControlRuleSetArgs)

            __props__.__dict__["grant_rules"] = grant_rules
            __props__.__dict__["name"] = name
            __props__.__dict__["etag"] = None
        super(AccessControlRuleSet, __self__).__init__(
            'databricks:index/accessControlRuleSet:AccessControlRuleSet',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            etag: Optional[pulumi.Input[_builtins.str]] = None,
            grant_rules: Optional[pulumi.Input[Sequence[pulumi.Input[Union['AccessControlRuleSetGrantRuleArgs', 'AccessControlRuleSetGrantRuleArgsDict']]]]] = None,
            name: Optional[pulumi.Input[_builtins.str]] = None) -> 'AccessControlRuleSet':
        """
        Get an existing AccessControlRuleSet resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['AccessControlRuleSetGrantRuleArgs', 'AccessControlRuleSetGrantRuleArgsDict']]]] grant_rules: The access control rules to be granted by this rule set, consisting of a set of principals and roles to be granted to them.
               
               !> Name uniquely identifies a rule set resource. Ensure all the grant_rules blocks for a rule set name are present in one `AccessControlRuleSet` resource block. Otherwise, after applying changes, users might lose their role assignment even if that was not intended.
        :param pulumi.Input[_builtins.str] name: Unique identifier of a rule set. The name determines the resource to which the rule set applies. **Changing the name recreates the resource!**. Currently, only default rule sets are supported. The following rule set formats are supported:
               * `accounts/{account_id}/ruleSets/default` - account-level access control.
               * `accounts/{account_id}/servicePrincipals/{service_principal_application_id}/ruleSets/default` - access control for a specific service principal.
               * `accounts/{account_id}/groups/{group_id}/ruleSets/default` - access control for a specific group.
               * `accounts/{account_id}/budgetPolicies/{budget_policy_id}/ruleSets/default` - access control for a specific budget policy.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _AccessControlRuleSetState.__new__(_AccessControlRuleSetState)

        __props__.__dict__["etag"] = etag
        __props__.__dict__["grant_rules"] = grant_rules
        __props__.__dict__["name"] = name
        return AccessControlRuleSet(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter
    def etag(self) -> pulumi.Output[_builtins.str]:
        return pulumi.get(self, "etag")

    @_builtins.property
    @pulumi.getter(name="grantRules")
    def grant_rules(self) -> pulumi.Output[Optional[Sequence['outputs.AccessControlRuleSetGrantRule']]]:
        """
        The access control rules to be granted by this rule set, consisting of a set of principals and roles to be granted to them.

        !> Name uniquely identifies a rule set resource. Ensure all the grant_rules blocks for a rule set name are present in one `AccessControlRuleSet` resource block. Otherwise, after applying changes, users might lose their role assignment even if that was not intended.
        """
        return pulumi.get(self, "grant_rules")

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Output[_builtins.str]:
        """
        Unique identifier of a rule set. The name determines the resource to which the rule set applies. **Changing the name recreates the resource!**. Currently, only default rule sets are supported. The following rule set formats are supported:
        * `accounts/{account_id}/ruleSets/default` - account-level access control.
        * `accounts/{account_id}/servicePrincipals/{service_principal_application_id}/ruleSets/default` - access control for a specific service principal.
        * `accounts/{account_id}/groups/{group_id}/ruleSets/default` - access control for a specific group.
        * `accounts/{account_id}/budgetPolicies/{budget_policy_id}/ruleSets/default` - access control for a specific budget policy.
        """
        return pulumi.get(self, "name")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetRfaAccessRequestDestinationsResult',
    'AwaitableGetRfaAccessRequestDestinationsResult',
    'get_rfa_access_request_destinations',
    'get_rfa_access_request_destinations_output',
]

@pulumi.output_type
class GetRfaAccessRequestDestinationsResult:
    """
    A collection of values returned by getRfaAccessRequestDestinations.
    """
    def __init__(__self__, are_any_destinations_hidden=None, destinations=None, id=None, securable=None):
        if are_any_destinations_hidden and not isinstance(are_any_destinations_hidden, bool):
            raise TypeError("Expected argument 'are_any_destinations_hidden' to be a bool")
        pulumi.set(__self__, "are_any_destinations_hidden", are_any_destinations_hidden)
        if destinations and not isinstance(destinations, list):
            raise TypeError("Expected argument 'destinations' to be a list")
        pulumi.set(__self__, "destinations", destinations)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if securable and not isinstance(securable, dict):
            raise TypeError("Expected argument 'securable' to be a dict")
        pulumi.set(__self__, "securable", securable)

    @_builtins.property
    @pulumi.getter(name="areAnyDestinationsHidden")
    def are_any_destinations_hidden(self) -> _builtins.bool:
        """
        (boolean) - Indicates whether any destinations are hidden from the caller due to a lack of permissions.
        This value is true if the caller does not have permission to see all destinations
        """
        return pulumi.get(self, "are_any_destinations_hidden")

    @_builtins.property
    @pulumi.getter
    def destinations(self) -> Sequence['outputs.GetRfaAccessRequestDestinationsDestinationResult']:
        """
        (list of NotificationDestination) - The access request destinations for the securable
        """
        return pulumi.get(self, "destinations")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def securable(self) -> 'outputs.GetRfaAccessRequestDestinationsSecurableResult':
        """
        (Securable) - The securable for which the access request destinations are being retrieved
        """
        return pulumi.get(self, "securable")


class AwaitableGetRfaAccessRequestDestinationsResult(GetRfaAccessRequestDestinationsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetRfaAccessRequestDestinationsResult(
            are_any_destinations_hidden=self.are_any_destinations_hidden,
            destinations=self.destinations,
            id=self.id,
            securable=self.securable)


def get_rfa_access_request_destinations(opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetRfaAccessRequestDestinationsResult:
    """
    [![Public Preview](https://img.shields.io/badge/Release_Stage-Public_Preview-yellowgreen)](https://docs.databricks.com/aws/en/release-notes/release-types)

    This data source can be used to get the Request for Access (RFA) access request destinations for a specific securable object.

    ## Example Usage

    Referring to RFA access request destinations by securable type and full name:
    """
    __args__ = dict()
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('databricks:index/getRfaAccessRequestDestinations:getRfaAccessRequestDestinations', __args__, opts=opts, typ=GetRfaAccessRequestDestinationsResult).value

    return AwaitableGetRfaAccessRequestDestinationsResult(
        are_any_destinations_hidden=pulumi.get(__ret__, 'are_any_destinations_hidden'),
        destinations=pulumi.get(__ret__, 'destinations'),
        id=pulumi.get(__ret__, 'id'),
        securable=pulumi.get(__ret__, 'securable'))
def get_rfa_access_request_destinations_output(opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetRfaAccessRequestDestinationsResult]:
    """
    [![Public Preview](https://img.shields.io/badge/Release_Stage-Public_Preview-yellowgreen)](https://docs.databricks.com/aws/en/release-notes/release-types)

    This data source can be used to get the Request for Access (RFA) access request destinations for a specific securable object.

    ## Example Usage

    Referring to RFA access request destinations by securable type and full name:
    """
    __args__ = dict()
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('databricks:index/getRfaAccessRequestDestinations:getRfaAccessRequestDestinations', __args__, opts=opts, typ=GetRfaAccessRequestDestinationsResult)
    return __ret__.apply(lambda __response__: GetRfaAccessRequestDestinationsResult(
        are_any_destinations_hidden=pulumi.get(__response__, 'are_any_destinations_hidden'),
        destinations=pulumi.get(__response__, 'destinations'),
        id=pulumi.get(__response__, 'id'),
        securable=pulumi.get(__response__, 'securable')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetAlertV2Result',
    'AwaitableGetAlertV2Result',
    'get_alert_v2',
    'get_alert_v2_output',
]

@pulumi.output_type
class GetAlertV2Result:
    """
    A collection of values returned by getAlertV2.
    """
    def __init__(__self__, create_time=None, custom_description=None, custom_summary=None, display_name=None, effective_run_as=None, evaluation=None, id=None, lifecycle_state=None, owner_user_name=None, parent_path=None, query_text=None, run_as=None, run_as_user_name=None, schedule=None, update_time=None, warehouse_id=None):
        if create_time and not isinstance(create_time, str):
            raise TypeError("Expected argument 'create_time' to be a str")
        pulumi.set(__self__, "create_time", create_time)
        if custom_description and not isinstance(custom_description, str):
            raise TypeError("Expected argument 'custom_description' to be a str")
        pulumi.set(__self__, "custom_description", custom_description)
        if custom_summary and not isinstance(custom_summary, str):
            raise TypeError("Expected argument 'custom_summary' to be a str")
        pulumi.set(__self__, "custom_summary", custom_summary)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if effective_run_as and not isinstance(effective_run_as, dict):
            raise TypeError("Expected argument 'effective_run_as' to be a dict")
        pulumi.set(__self__, "effective_run_as", effective_run_as)
        if evaluation and not isinstance(evaluation, dict):
            raise TypeError("Expected argument 'evaluation' to be a dict")
        pulumi.set(__self__, "evaluation", evaluation)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if lifecycle_state and not isinstance(lifecycle_state, str):
            raise TypeError("Expected argument 'lifecycle_state' to be a str")
        pulumi.set(__self__, "lifecycle_state", lifecycle_state)
        if owner_user_name and not isinstance(owner_user_name, str):
            raise TypeError("Expected argument 'owner_user_name' to be a str")
        pulumi.set(__self__, "owner_user_name", owner_user_name)
        if parent_path and not isinstance(parent_path, str):
            raise TypeError("Expected argument 'parent_path' to be a str")
        pulumi.set(__self__, "parent_path", parent_path)
        if query_text and not isinstance(query_text, str):
            raise TypeError("Expected argument 'query_text' to be a str")
        pulumi.set(__self__, "query_text", query_text)
        if run_as and not isinstance(run_as, dict):
            raise TypeError("Expected argument 'run_as' to be a dict")
        pulumi.set(__self__, "run_as", run_as)
        if run_as_user_name and not isinstance(run_as_user_name, str):
            raise TypeError("Expected argument 'run_as_user_name' to be a str")
        pulumi.set(__self__, "run_as_user_name", run_as_user_name)
        if schedule and not isinstance(schedule, dict):
            raise TypeError("Expected argument 'schedule' to be a dict")
        pulumi.set(__self__, "schedule", schedule)
        if update_time and not isinstance(update_time, str):
            raise TypeError("Expected argument 'update_time' to be a str")
        pulumi.set(__self__, "update_time", update_time)
        if warehouse_id and not isinstance(warehouse_id, str):
            raise TypeError("Expected argument 'warehouse_id' to be a str")
        pulumi.set(__self__, "warehouse_id", warehouse_id)

    @_builtins.property
    @pulumi.getter(name="createTime")
    def create_time(self) -> _builtins.str:
        """
        (string) - The timestamp indicating when the alert was created
        """
        return pulumi.get(self, "create_time")

    @_builtins.property
    @pulumi.getter(name="customDescription")
    def custom_description(self) -> _builtins.str:
        """
        (string) - Custom description for the alert. support mustache template
        """
        return pulumi.get(self, "custom_description")

    @_builtins.property
    @pulumi.getter(name="customSummary")
    def custom_summary(self) -> _builtins.str:
        """
        (string) - Custom summary for the alert. support mustache template
        """
        return pulumi.get(self, "custom_summary")

    @_builtins.property
    @pulumi.getter(name="displayName")
    def display_name(self) -> _builtins.str:
        """
        (string) - The display name of the alert
        """
        return pulumi.get(self, "display_name")

    @_builtins.property
    @pulumi.getter(name="effectiveRunAs")
    def effective_run_as(self) -> 'outputs.GetAlertV2EffectiveRunAsResult':
        """
        (AlertV2RunAs) - The actual identity that will be used to execute the alert.
        This is an output-only field that shows the resolved run-as identity after applying
        permissions and defaults
        """
        return pulumi.get(self, "effective_run_as")

    @_builtins.property
    @pulumi.getter
    def evaluation(self) -> 'outputs.GetAlertV2EvaluationResult':
        """
        (AlertV2Evaluation)
        """
        return pulumi.get(self, "evaluation")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        (string) - UUID identifying the alert
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="lifecycleState")
    def lifecycle_state(self) -> _builtins.str:
        """
        (string) - Indicates whether the query is trashed. Possible values are: `ACTIVE`, `DELETED`
        """
        return pulumi.get(self, "lifecycle_state")

    @_builtins.property
    @pulumi.getter(name="ownerUserName")
    def owner_user_name(self) -> _builtins.str:
        """
        (string) - The owner's username. This field is set to "Unavailable" if the user has been deleted
        """
        return pulumi.get(self, "owner_user_name")

    @_builtins.property
    @pulumi.getter(name="parentPath")
    def parent_path(self) -> _builtins.str:
        """
        (string) - The workspace path of the folder containing the alert. Can only be set on create, and cannot be updated
        """
        return pulumi.get(self, "parent_path")

    @_builtins.property
    @pulumi.getter(name="queryText")
    def query_text(self) -> _builtins.str:
        """
        (string) - Text of the query to be run
        """
        return pulumi.get(self, "query_text")

    @_builtins.property
    @pulumi.getter(name="runAs")
    def run_as(self) -> 'outputs.GetAlertV2RunAsResult':
        """
        (AlertV2RunAs) - Specifies the identity that will be used to run the alert.
        This field allows you to configure alerts to run as a specific user or service principal.
        - For user identity: Set `user_name` to the email of an active workspace user. Users can only set this to their own email.
        - For service principal: Set `service_principal_name` to the application ID. Requires the `servicePrincipal/user` role.
        If not specified, the alert will run as the request user
        """
        return pulumi.get(self, "run_as")

    @_builtins.property
    @pulumi.getter(name="runAsUserName")
    def run_as_user_name(self) -> _builtins.str:
        """
        (string, deprecated) - The run as username or application ID of service principal.
        On Create and Update, this field can be set to application ID of an active service principal. Setting this field requires the servicePrincipal/user role.
        Deprecated: Use `run_as` field instead. This field will be removed in a future release
        """
        return pulumi.get(self, "run_as_user_name")

    @_builtins.property
    @pulumi.getter
    def schedule(self) -> 'outputs.GetAlertV2ScheduleResult':
        """
        (CronSchedule)
        """
        return pulumi.get(self, "schedule")

    @_builtins.property
    @pulumi.getter(name="updateTime")
    def update_time(self) -> _builtins.str:
        """
        (string) - The timestamp indicating when the alert was updated
        """
        return pulumi.get(self, "update_time")

    @_builtins.property
    @pulumi.getter(name="warehouseId")
    def warehouse_id(self) -> _builtins.str:
        """
        (string) - ID of the SQL warehouse attached to the alert
        """
        return pulumi.get(self, "warehouse_id")


class AwaitableGetAlertV2Result(GetAlertV2Result):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetAlertV2Result(
            create_time=self.create_time,
            custom_description=self.custom_description,
            custom_summary=self.custom_summary,
            display_name=self.display_name,
            effective_run_as=self.effective_run_as,
            evaluation=self.evaluation,
            id=self.id,
            lifecycle_state=self.lifecycle_state,
            owner_user_name=self.owner_user_name,
            parent_path=self.parent_path,
            query_text=self.query_text,
            run_as=self.run_as,
            run_as_user_name=self.run_as_user_name,
            schedule=self.schedule,
            update_time=self.update_time,
            warehouse_id=self.warehouse_id)


def get_alert_v2(id: Optional[_builtins.str] = None,
                 opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetAlertV2Result:
    """
    [![Public Beta](https://img.shields.io/badge/Release_Stage-Public_Beta-orange)](https://docs.databricks.com/aws/en/release-notes/release-types)

    The SQL Alert v2 data source allows you to retrieve detailed information about a specific alert in Databricks SQL. This data source provides access to all alert properties, including its configuration, evaluation criteria, notification settings, and schedule.

    You can use this data source to:
    - Retrieve alert details for reference in other resources
    - Check the current state and configuration of an alert
    - Verify notification settings and subscribers
    - Examine the schedule configuration

    ## Example Usage

    ### Retrieve Alert by ID
    This example retrieves a specific alert by its ID:

    ```python
    import pulumi
    import pulumi_databricks as databricks

    this = databricks.get_alert_v2(id="123")
    ```


    :param _builtins.str id: UUID identifying the alert
    """
    __args__ = dict()
    __args__['id'] = id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('databricks:index/getAlertV2:getAlertV2', __args__, opts=opts, typ=GetAlertV2Result).value

    return AwaitableGetAlertV2Result(
        create_time=pulumi.get(__ret__, 'create_time'),
        custom_description=pulumi.get(__ret__, 'custom_description'),
        custom_summary=pulumi.get(__ret__, 'custom_summary'),
        display_name=pulumi.get(__ret__, 'display_name'),
        effective_run_as=pulumi.get(__ret__, 'effective_run_as'),
        evaluation=pulumi.get(__ret__, 'evaluation'),
        id=pulumi.get(__ret__, 'id'),
        lifecycle_state=pulumi.get(__ret__, 'lifecycle_state'),
        owner_user_name=pulumi.get(__ret__, 'owner_user_name'),
        parent_path=pulumi.get(__ret__, 'parent_path'),
        query_text=pulumi.get(__ret__, 'query_text'),
        run_as=pulumi.get(__ret__, 'run_as'),
        run_as_user_name=pulumi.get(__ret__, 'run_as_user_name'),
        schedule=pulumi.get(__ret__, 'schedule'),
        update_time=pulumi.get(__ret__, 'update_time'),
        warehouse_id=pulumi.get(__ret__, 'warehouse_id'))
def get_alert_v2_output(id: Optional[pulumi.Input[_builtins.str]] = None,
                        opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetAlertV2Result]:
    """
    [![Public Beta](https://img.shields.io/badge/Release_Stage-Public_Beta-orange)](https://docs.databricks.com/aws/en/release-notes/release-types)

    The SQL Alert v2 data source allows you to retrieve detailed information about a specific alert in Databricks SQL. This data source provides access to all alert properties, including its configuration, evaluation criteria, notification settings, and schedule.

    You can use this data source to:
    - Retrieve alert details for reference in other resources
    - Check the current state and configuration of an alert
    - Verify notification settings and subscribers
    - Examine the schedule configuration

    ## Example Usage

    ### Retrieve Alert by ID
    This example retrieves a specific alert by its ID:

    ```python
    import pulumi
    import pulumi_databricks as databricks

    this = databricks.get_alert_v2(id="123")
    ```


    :param _builtins.str id: UUID identifying the alert
    """
    __args__ = dict()
    __args__['id'] = id
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('databricks:index/getAlertV2:getAlertV2', __args__, opts=opts, typ=GetAlertV2Result)
    return __ret__.apply(lambda __response__: GetAlertV2Result(
        create_time=pulumi.get(__response__, 'create_time'),
        custom_description=pulumi.get(__response__, 'custom_description'),
        custom_summary=pulumi.get(__response__, 'custom_summary'),
        display_name=pulumi.get(__response__, 'display_name'),
        effective_run_as=pulumi.get(__response__, 'effective_run_as'),
        evaluation=pulumi.get(__response__, 'evaluation'),
        id=pulumi.get(__response__, 'id'),
        lifecycle_state=pulumi.get(__response__, 'lifecycle_state'),
        owner_user_name=pulumi.get(__response__, 'owner_user_name'),
        parent_path=pulumi.get(__response__, 'parent_path'),
        query_text=pulumi.get(__response__, 'query_text'),
        run_as=pulumi.get(__response__, 'run_as'),
        run_as_user_name=pulumi.get(__response__, 'run_as_user_name'),
        schedule=pulumi.get(__response__, 'schedule'),
        update_time=pulumi.get(__response__, 'update_time'),
        warehouse_id=pulumi.get(__response__, 'warehouse_id')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = [
    'GetAwsUnityCatalogPolicyResult',
    'AwaitableGetAwsUnityCatalogPolicyResult',
    'get_aws_unity_catalog_policy',
    'get_aws_unity_catalog_policy_output',
]

@pulumi.output_type
class GetAwsUnityCatalogPolicyResult:
    """
    A collection of values returned by getAwsUnityCatalogPolicy.
    """
    def __init__(__self__, aws_account_id=None, aws_partition=None, bucket_name=None, id=None, json=None, kms_name=None, role_name=None):
        if aws_account_id and not isinstance(aws_account_id, str):
            raise TypeError("Expected argument 'aws_account_id' to be a str")
        pulumi.set(__self__, "aws_account_id", aws_account_id)
        if aws_partition and not isinstance(aws_partition, str):
            raise TypeError("Expected argument 'aws_partition' to be a str")
        pulumi.set(__self__, "aws_partition", aws_partition)
        if bucket_name and not isinstance(bucket_name, str):
            raise TypeError("Expected argument 'bucket_name' to be a str")
        pulumi.set(__self__, "bucket_name", bucket_name)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if json and not isinstance(json, str):
            raise TypeError("Expected argument 'json' to be a str")
        pulumi.set(__self__, "json", json)
        if kms_name and not isinstance(kms_name, str):
            raise TypeError("Expected argument 'kms_name' to be a str")
        pulumi.set(__self__, "kms_name", kms_name)
        if role_name and not isinstance(role_name, str):
            raise TypeError("Expected argument 'role_name' to be a str")
        pulumi.set(__self__, "role_name", role_name)

    @_builtins.property
    @pulumi.getter(name="awsAccountId")
    def aws_account_id(self) -> _builtins.str:
        return pulumi.get(self, "aws_account_id")

    @_builtins.property
    @pulumi.getter(name="awsPartition")
    def aws_partition(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "aws_partition")

    @_builtins.property
    @pulumi.getter(name="bucketName")
    def bucket_name(self) -> _builtins.str:
        return pulumi.get(self, "bucket_name")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def json(self) -> _builtins.str:
        """
        AWS IAM Policy JSON document
        """
        return pulumi.get(self, "json")

    @_builtins.property
    @pulumi.getter(name="kmsName")
    def kms_name(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "kms_name")

    @_builtins.property
    @pulumi.getter(name="roleName")
    def role_name(self) -> _builtins.str:
        return pulumi.get(self, "role_name")


class AwaitableGetAwsUnityCatalogPolicyResult(GetAwsUnityCatalogPolicyResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetAwsUnityCatalogPolicyResult(
            aws_account_id=self.aws_account_id,
            aws_partition=self.aws_partition,
            bucket_name=self.bucket_name,
            id=self.id,
            json=self.json,
            kms_name=self.kms_name,
            role_name=self.role_name)


def get_aws_unity_catalog_policy(aws_account_id: Optional[_builtins.str] = None,
                                 aws_partition: Optional[_builtins.str] = None,
                                 bucket_name: Optional[_builtins.str] = None,
                                 kms_name: Optional[_builtins.str] = None,
                                 role_name: Optional[_builtins.str] = None,
                                 opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetAwsUnityCatalogPolicyResult:
    """
    This data source constructs the necessary AWS Unity Catalog policy for you.

    > This data source can be used with an account or workspace-level provider.

    > This data source has an evolving API, which may change in future versions of the provider. Please always consult [latest documentation](https://docs.databricks.com/data-governance/unity-catalog/get-started.html#configure-a-storage-bucket-and-iam-role-in-aws) in case of any questions.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_aws as aws
    import pulumi_databricks as databricks

    this = databricks.get_aws_unity_catalog_policy(aws_account_id=aws_account_id,
        bucket_name="databricks-bucket",
        role_name=f"{prefix}-uc-access",
        kms_name="arn:aws:kms:us-west-2:111122223333:key/databricks-kms")
    this_get_aws_unity_catalog_assume_role_policy = databricks.get_aws_unity_catalog_assume_role_policy(aws_account_id=aws_account_id,
        role_name=f"{prefix}-uc-access",
        external_id="12345")
    unity_metastore = aws.index.IamPolicy("unity_metastore",
        name=f{prefix}-unity-catalog-metastore-access-iam-policy,
        policy=this.json)
    metastore_data_access = aws.index.IamRole("metastore_data_access",
        name=f{prefix}-uc-access,
        assume_role_policy=this_get_aws_unity_catalog_assume_role_policy.json)
    metastore_data_access_iam_role_policy_attachment = aws.index.IamRolePolicyAttachment("metastore_data_access",
        role=metastore_data_access.name,
        policy_arn=unity_metastore.arn)
    ```


    :param _builtins.str aws_account_id: The Account ID of the current AWS account (not your Databricks account).
    :param _builtins.str aws_partition: AWS partition. The options are `aws`, `aws-us-gov`, or `aws-us-gov-dod`. Defaults to `aws`
    :param _builtins.str bucket_name: The name of the S3 bucket used as root storage location for [managed tables](https://docs.databricks.com/data-governance/unity-catalog/index.html#managed-table) in Unity Catalog.  The name must follow the [S3 bucket naming rules](https://docs.aws.amazon.com/AmazonS3/latest/userguide/bucketnamingrules.html).
    :param _builtins.str kms_name: If encryption is enabled, provide the ARN of the KMS key that encrypts the S3 bucket contents. If encryption is disabled, do not provide this argument.
    :param _builtins.str role_name: The name of the AWS IAM role that you created in the previous step in the [official documentation](https://docs.databricks.com/data-governance/unity-catalog/get-started.html#configure-a-storage-bucket-and-iam-role-in-aws).
    """
    __args__ = dict()
    __args__['awsAccountId'] = aws_account_id
    __args__['awsPartition'] = aws_partition
    __args__['bucketName'] = bucket_name
    __args__['kmsName'] = kms_name
    __args__['roleName'] = role_name
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('databricks:index/getAwsUnityCatalogPolicy:getAwsUnityCatalogPolicy', __args__, opts=opts, typ=GetAwsUnityCatalogPolicyResult).value

    return AwaitableGetAwsUnityCatalogPolicyResult(
        aws_account_id=pulumi.get(__ret__, 'aws_account_id'),
        aws_partition=pulumi.get(__ret__, 'aws_partition'),
        bucket_name=pulumi.get(__ret__, 'bucket_name'),
        id=pulumi.get(__ret__, 'id'),
        json=pulumi.get(__ret__, 'json'),
        kms_name=pulumi.get(__ret__, 'kms_name'),
        role_name=pulumi.get(__ret__, 'role_name'))
def get_aws_unity_catalog_policy_output(aws_account_id: Optional[pulumi.Input[_builtins.str]] = None,
                                        aws_partition: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                                        bucket_name: Optional[pulumi.Input[_builtins.str]] = None,
                                        kms_name: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                                        role_name: Optional[pulumi.Input[_builtins.str]] = None,
                                        opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetAwsUnityCatalogPolicyResult]:
    """
    This data source constructs the necessary AWS Unity Catalog policy for you.

    > This data source can be used with an account or workspace-level provider.

    > This data source has an evolving API, which may change in future versions of the provider. Please always consult [latest documentation](https://docs.databricks.com/data-governance/unity-catalog/get-started.html#configure-a-storage-bucket-and-iam-role-in-aws) in case of any questions.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_aws as aws
    import pulumi_databricks as databricks

    this = databricks.get_aws_unity_catalog_policy(aws_account_id=aws_account_id,
        bucket_name="databricks-bucket",
        role_name=f"{prefix}-uc-access",
        kms_name="arn:aws:kms:us-west-2:111122223333:key/databricks-kms")
    this_get_aws_unity_catalog_assume_role_policy = databricks.get_aws_unity_catalog_assume_role_policy(aws_account_id=aws_account_id,
        role_name=f"{prefix}-uc-access",
        external_id="12345")
    unity_metastore = aws.index.IamPolicy("unity_metastore",
        name=f{prefix}-unity-catalog-metastore-access-iam-policy,
        policy=this.json)
    metastore_data_access = aws.index.IamRole("metastore_data_access",
        name=f{prefix}-uc-access,
        assume_role_policy=this_get_aws_unity_catalog_assume_role_policy.json)
    metastore_data_access_iam_role_policy_attachment = aws.index.IamRolePolicyAttachment("metastore_data_access",
        role=metastore_data_access.name,
        policy_arn=unity_metastore.arn)
    ```


    :param _builtins.str aws_account_id: The Account ID of the current AWS account (not your Databricks account).
    :param _builtins.str aws_partition: AWS partition. The options are `aws`, `aws-us-gov`, or `aws-us-gov-dod`. Defaults to `aws`
    :param _builtins.str bucket_name: The name of the S3 bucket used as root storage location for [managed tables](https://docs.databricks.com/data-governance/unity-catalog/index.html#managed-table) in Unity Catalog.  The name must follow the [S3 bucket naming rules](https://docs.aws.amazon.com/AmazonS3/latest/userguide/bucketnamingrules.html).
    :param _builtins.str kms_name: If encryption is enabled, provide the ARN of the KMS key that encrypts the S3 bucket contents. If encryption is disabled, do not provide this argument.
    :param _builtins.str role_name: The name of the AWS IAM role that you created in the previous step in the [official documentation](https://docs.databricks.com/data-governance/unity-catalog/get-started.html#configure-a-storage-bucket-and-iam-role-in-aws).
    """
    __args__ = dict()
    __args__['awsAccountId'] = aws_account_id
    __args__['awsPartition'] = aws_partition
    __args__['bucketName'] = bucket_name
    __args__['kmsName'] = kms_name
    __args__['roleName'] = role_name
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('databricks:index/getAwsUnityCatalogPolicy:getAwsUnityCatalogPolicy', __args__, opts=opts, typ=GetAwsUnityCatalogPolicyResult)
    return __ret__.apply(lambda __response__: GetAwsUnityCatalogPolicyResult(
        aws_account_id=pulumi.get(__response__, 'aws_account_id'),
        aws_partition=pulumi.get(__response__, 'aws_partition'),
        bucket_name=pulumi.get(__response__, 'bucket_name'),
        id=pulumi.get(__response__, 'id'),
        json=pulumi.get(__response__, 'json'),
        kms_name=pulumi.get(__response__, 'kms_name'),
        role_name=pulumi.get(__response__, 'role_name')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['ModelServingArgs', 'ModelServing']

@pulumi.input_type
class ModelServingArgs:
    def __init__(__self__, *,
                 ai_gateway: Optional[pulumi.Input['ModelServingAiGatewayArgs']] = None,
                 budget_policy_id: Optional[pulumi.Input[_builtins.str]] = None,
                 config: Optional[pulumi.Input['ModelServingConfigArgs']] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 email_notifications: Optional[pulumi.Input['ModelServingEmailNotificationsArgs']] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 rate_limits: Optional[pulumi.Input[Sequence[pulumi.Input['ModelServingRateLimitArgs']]]] = None,
                 route_optimized: Optional[pulumi.Input[_builtins.bool]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input['ModelServingTagArgs']]]] = None):
        """
        The set of arguments for constructing a ModelServing resource.
        :param pulumi.Input['ModelServingAiGatewayArgs'] ai_gateway: A block with AI Gateway configuration for the serving endpoint. *Note: only external model endpoints are supported as of now.*
        :param pulumi.Input[_builtins.str] budget_policy_id: The Budget Policy ID set for this serving endpoint.
        :param pulumi.Input['ModelServingConfigArgs'] config: The model serving endpoint configuration. This is optional and can be added and modified after creation. If `config` was provided in a previous apply but is not provided in the current apply, no change to the model serving endpoint will occur. To recreate the model serving endpoint without the `config` block, the model serving endpoint must be destroyed and recreated.
        :param pulumi.Input[_builtins.str] description: The description of the model serving endpoint.
        :param pulumi.Input['ModelServingEmailNotificationsArgs'] email_notifications: A block with Email notification setting.
        :param pulumi.Input[_builtins.str] name: The name of the model serving endpoint. This field is required and must be unique across a workspace. An endpoint name can consist of alphanumeric characters, dashes, and underscores. NOTE: Changing this name will delete the existing endpoint and create a new endpoint with the updated name.
        :param pulumi.Input[Sequence[pulumi.Input['ModelServingRateLimitArgs']]] rate_limits: A list of rate limit blocks to be applied to the serving endpoint. *Note: only external and foundation model endpoints are supported as of now.*
        :param pulumi.Input[_builtins.bool] route_optimized: A boolean enabling route optimization for the endpoint. *Note: only available for custom models.*
        :param pulumi.Input[Sequence[pulumi.Input['ModelServingTagArgs']]] tags: Tags to be attached to the serving endpoint and automatically propagated to billing logs.
        """
        if ai_gateway is not None:
            pulumi.set(__self__, "ai_gateway", ai_gateway)
        if budget_policy_id is not None:
            pulumi.set(__self__, "budget_policy_id", budget_policy_id)
        if config is not None:
            pulumi.set(__self__, "config", config)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if email_notifications is not None:
            pulumi.set(__self__, "email_notifications", email_notifications)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if rate_limits is not None:
            warnings.warn("""Please use AI Gateway to manage rate limits.""", DeprecationWarning)
            pulumi.log.warn("""rate_limits is deprecated: Please use AI Gateway to manage rate limits.""")
        if rate_limits is not None:
            pulumi.set(__self__, "rate_limits", rate_limits)
        if route_optimized is not None:
            pulumi.set(__self__, "route_optimized", route_optimized)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)

    @_builtins.property
    @pulumi.getter(name="aiGateway")
    def ai_gateway(self) -> Optional[pulumi.Input['ModelServingAiGatewayArgs']]:
        """
        A block with AI Gateway configuration for the serving endpoint. *Note: only external model endpoints are supported as of now.*
        """
        return pulumi.get(self, "ai_gateway")

    @ai_gateway.setter
    def ai_gateway(self, value: Optional[pulumi.Input['ModelServingAiGatewayArgs']]):
        pulumi.set(self, "ai_gateway", value)

    @_builtins.property
    @pulumi.getter(name="budgetPolicyId")
    def budget_policy_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The Budget Policy ID set for this serving endpoint.
        """
        return pulumi.get(self, "budget_policy_id")

    @budget_policy_id.setter
    def budget_policy_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "budget_policy_id", value)

    @_builtins.property
    @pulumi.getter
    def config(self) -> Optional[pulumi.Input['ModelServingConfigArgs']]:
        """
        The model serving endpoint configuration. This is optional and can be added and modified after creation. If `config` was provided in a previous apply but is not provided in the current apply, no change to the model serving endpoint will occur. To recreate the model serving endpoint without the `config` block, the model serving endpoint must be destroyed and recreated.
        """
        return pulumi.get(self, "config")

    @config.setter
    def config(self, value: Optional[pulumi.Input['ModelServingConfigArgs']]):
        pulumi.set(self, "config", value)

    @_builtins.property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The description of the model serving endpoint.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "description", value)

    @_builtins.property
    @pulumi.getter(name="emailNotifications")
    def email_notifications(self) -> Optional[pulumi.Input['ModelServingEmailNotificationsArgs']]:
        """
        A block with Email notification setting.
        """
        return pulumi.get(self, "email_notifications")

    @email_notifications.setter
    def email_notifications(self, value: Optional[pulumi.Input['ModelServingEmailNotificationsArgs']]):
        pulumi.set(self, "email_notifications", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the model serving endpoint. This field is required and must be unique across a workspace. An endpoint name can consist of alphanumeric characters, dashes, and underscores. NOTE: Changing this name will delete the existing endpoint and create a new endpoint with the updated name.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="rateLimits")
    @_utilities.deprecated("""Please use AI Gateway to manage rate limits.""")
    def rate_limits(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ModelServingRateLimitArgs']]]]:
        """
        A list of rate limit blocks to be applied to the serving endpoint. *Note: only external and foundation model endpoints are supported as of now.*
        """
        return pulumi.get(self, "rate_limits")

    @rate_limits.setter
    def rate_limits(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ModelServingRateLimitArgs']]]]):
        pulumi.set(self, "rate_limits", value)

    @_builtins.property
    @pulumi.getter(name="routeOptimized")
    def route_optimized(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        A boolean enabling route optimization for the endpoint. *Note: only available for custom models.*
        """
        return pulumi.get(self, "route_optimized")

    @route_optimized.setter
    def route_optimized(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "route_optimized", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ModelServingTagArgs']]]]:
        """
        Tags to be attached to the serving endpoint and automatically propagated to billing logs.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ModelServingTagArgs']]]]):
        pulumi.set(self, "tags", value)


@pulumi.input_type
class _ModelServingState:
    def __init__(__self__, *,
                 ai_gateway: Optional[pulumi.Input['ModelServingAiGatewayArgs']] = None,
                 budget_policy_id: Optional[pulumi.Input[_builtins.str]] = None,
                 config: Optional[pulumi.Input['ModelServingConfigArgs']] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 email_notifications: Optional[pulumi.Input['ModelServingEmailNotificationsArgs']] = None,
                 endpoint_url: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 rate_limits: Optional[pulumi.Input[Sequence[pulumi.Input['ModelServingRateLimitArgs']]]] = None,
                 route_optimized: Optional[pulumi.Input[_builtins.bool]] = None,
                 serving_endpoint_id: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input['ModelServingTagArgs']]]] = None):
        """
        Input properties used for looking up and filtering ModelServing resources.
        :param pulumi.Input['ModelServingAiGatewayArgs'] ai_gateway: A block with AI Gateway configuration for the serving endpoint. *Note: only external model endpoints are supported as of now.*
        :param pulumi.Input[_builtins.str] budget_policy_id: The Budget Policy ID set for this serving endpoint.
        :param pulumi.Input['ModelServingConfigArgs'] config: The model serving endpoint configuration. This is optional and can be added and modified after creation. If `config` was provided in a previous apply but is not provided in the current apply, no change to the model serving endpoint will occur. To recreate the model serving endpoint without the `config` block, the model serving endpoint must be destroyed and recreated.
        :param pulumi.Input[_builtins.str] description: The description of the model serving endpoint.
        :param pulumi.Input['ModelServingEmailNotificationsArgs'] email_notifications: A block with Email notification setting.
        :param pulumi.Input[_builtins.str] endpoint_url: Invocation url of the endpoint.
        :param pulumi.Input[_builtins.str] name: The name of the model serving endpoint. This field is required and must be unique across a workspace. An endpoint name can consist of alphanumeric characters, dashes, and underscores. NOTE: Changing this name will delete the existing endpoint and create a new endpoint with the updated name.
        :param pulumi.Input[Sequence[pulumi.Input['ModelServingRateLimitArgs']]] rate_limits: A list of rate limit blocks to be applied to the serving endpoint. *Note: only external and foundation model endpoints are supported as of now.*
        :param pulumi.Input[_builtins.bool] route_optimized: A boolean enabling route optimization for the endpoint. *Note: only available for custom models.*
        :param pulumi.Input[_builtins.str] serving_endpoint_id: Unique identifier of the serving endpoint primarily used to set permissions and refer to this instance for other operations.
        :param pulumi.Input[Sequence[pulumi.Input['ModelServingTagArgs']]] tags: Tags to be attached to the serving endpoint and automatically propagated to billing logs.
        """
        if ai_gateway is not None:
            pulumi.set(__self__, "ai_gateway", ai_gateway)
        if budget_policy_id is not None:
            pulumi.set(__self__, "budget_policy_id", budget_policy_id)
        if config is not None:
            pulumi.set(__self__, "config", config)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if email_notifications is not None:
            pulumi.set(__self__, "email_notifications", email_notifications)
        if endpoint_url is not None:
            pulumi.set(__self__, "endpoint_url", endpoint_url)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if rate_limits is not None:
            warnings.warn("""Please use AI Gateway to manage rate limits.""", DeprecationWarning)
            pulumi.log.warn("""rate_limits is deprecated: Please use AI Gateway to manage rate limits.""")
        if rate_limits is not None:
            pulumi.set(__self__, "rate_limits", rate_limits)
        if route_optimized is not None:
            pulumi.set(__self__, "route_optimized", route_optimized)
        if serving_endpoint_id is not None:
            pulumi.set(__self__, "serving_endpoint_id", serving_endpoint_id)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)

    @_builtins.property
    @pulumi.getter(name="aiGateway")
    def ai_gateway(self) -> Optional[pulumi.Input['ModelServingAiGatewayArgs']]:
        """
        A block with AI Gateway configuration for the serving endpoint. *Note: only external model endpoints are supported as of now.*
        """
        return pulumi.get(self, "ai_gateway")

    @ai_gateway.setter
    def ai_gateway(self, value: Optional[pulumi.Input['ModelServingAiGatewayArgs']]):
        pulumi.set(self, "ai_gateway", value)

    @_builtins.property
    @pulumi.getter(name="budgetPolicyId")
    def budget_policy_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The Budget Policy ID set for this serving endpoint.
        """
        return pulumi.get(self, "budget_policy_id")

    @budget_policy_id.setter
    def budget_policy_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "budget_policy_id", value)

    @_builtins.property
    @pulumi.getter
    def config(self) -> Optional[pulumi.Input['ModelServingConfigArgs']]:
        """
        The model serving endpoint configuration. This is optional and can be added and modified after creation. If `config` was provided in a previous apply but is not provided in the current apply, no change to the model serving endpoint will occur. To recreate the model serving endpoint without the `config` block, the model serving endpoint must be destroyed and recreated.
        """
        return pulumi.get(self, "config")

    @config.setter
    def config(self, value: Optional[pulumi.Input['ModelServingConfigArgs']]):
        pulumi.set(self, "config", value)

    @_builtins.property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The description of the model serving endpoint.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "description", value)

    @_builtins.property
    @pulumi.getter(name="emailNotifications")
    def email_notifications(self) -> Optional[pulumi.Input['ModelServingEmailNotificationsArgs']]:
        """
        A block with Email notification setting.
        """
        return pulumi.get(self, "email_notifications")

    @email_notifications.setter
    def email_notifications(self, value: Optional[pulumi.Input['ModelServingEmailNotificationsArgs']]):
        pulumi.set(self, "email_notifications", value)

    @_builtins.property
    @pulumi.getter(name="endpointUrl")
    def endpoint_url(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Invocation url of the endpoint.
        """
        return pulumi.get(self, "endpoint_url")

    @endpoint_url.setter
    def endpoint_url(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "endpoint_url", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the model serving endpoint. This field is required and must be unique across a workspace. An endpoint name can consist of alphanumeric characters, dashes, and underscores. NOTE: Changing this name will delete the existing endpoint and create a new endpoint with the updated name.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="rateLimits")
    @_utilities.deprecated("""Please use AI Gateway to manage rate limits.""")
    def rate_limits(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ModelServingRateLimitArgs']]]]:
        """
        A list of rate limit blocks to be applied to the serving endpoint. *Note: only external and foundation model endpoints are supported as of now.*
        """
        return pulumi.get(self, "rate_limits")

    @rate_limits.setter
    def rate_limits(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ModelServingRateLimitArgs']]]]):
        pulumi.set(self, "rate_limits", value)

    @_builtins.property
    @pulumi.getter(name="routeOptimized")
    def route_optimized(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        A boolean enabling route optimization for the endpoint. *Note: only available for custom models.*
        """
        return pulumi.get(self, "route_optimized")

    @route_optimized.setter
    def route_optimized(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "route_optimized", value)

    @_builtins.property
    @pulumi.getter(name="servingEndpointId")
    def serving_endpoint_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Unique identifier of the serving endpoint primarily used to set permissions and refer to this instance for other operations.
        """
        return pulumi.get(self, "serving_endpoint_id")

    @serving_endpoint_id.setter
    def serving_endpoint_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "serving_endpoint_id", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ModelServingTagArgs']]]]:
        """
        Tags to be attached to the serving endpoint and automatically propagated to billing logs.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ModelServingTagArgs']]]]):
        pulumi.set(self, "tags", value)


@pulumi.type_token("databricks:index/modelServing:ModelServing")
class ModelServing(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 ai_gateway: Optional[pulumi.Input[Union['ModelServingAiGatewayArgs', 'ModelServingAiGatewayArgsDict']]] = None,
                 budget_policy_id: Optional[pulumi.Input[_builtins.str]] = None,
                 config: Optional[pulumi.Input[Union['ModelServingConfigArgs', 'ModelServingConfigArgsDict']]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 email_notifications: Optional[pulumi.Input[Union['ModelServingEmailNotificationsArgs', 'ModelServingEmailNotificationsArgsDict']]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 rate_limits: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ModelServingRateLimitArgs', 'ModelServingRateLimitArgsDict']]]]] = None,
                 route_optimized: Optional[pulumi.Input[_builtins.bool]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ModelServingTagArgs', 'ModelServingTagArgsDict']]]]] = None,
                 __props__=None):
        """
        This resource allows you to manage [Model Serving](https://docs.databricks.com/machine-learning/model-serving/index.html) endpoints in Databricks, including custom models, external models, and foundation models. For newer foundation models, including Llama 4, please use the ModelServingProvisionedThroughput resource.

        > This resource can only be used with a workspace-level provider!

        > If you replace `served_models` with `served_entities` in an existing serving endpoint, the serving endpoint will briefly go into an update state (~30 seconds) and increment the config version.

        ## Example Usage

        Creating a CPU serving endpoint

        ```python
        import pulumi
        import pulumi_databricks as databricks

        this = databricks.ModelServing("this",
            name="ads-serving-endpoint",
            config={
                "served_entities": [
                    {
                        "name": "prod_model",
                        "entity_name": "ads-model",
                        "entity_version": "2",
                        "workload_size": "Small",
                        "scale_to_zero_enabled": True,
                    },
                    {
                        "name": "candidate_model",
                        "entity_name": "ads-model",
                        "entity_version": "4",
                        "workload_size": "Small",
                        "scale_to_zero_enabled": False,
                    },
                ],
                "traffic_config": {
                    "routes": [
                        {
                            "served_model_name": "prod_model",
                            "traffic_percentage": 90,
                        },
                        {
                            "served_model_name": "candidate_model",
                            "traffic_percentage": 10,
                        },
                    ],
                },
            })
        ```

        Creating a Foundation Model endpoint

        ```python
        import pulumi
        import pulumi_databricks as databricks

        llama = databricks.ModelServing("llama",
            name="llama_3_2_3b_instruct",
            ai_gateway={
                "usage_tracking_config": {
                    "enabled": True,
                },
            },
            config={
                "served_entities": [{
                    "name": "meta_llama_v3_2_3b_instruct-3",
                    "entity_name": "system.ai.llama_v3_2_3b_instruct",
                    "entity_version": "2",
                    "scale_to_zero_enabled": True,
                    "max_provisioned_throughput": 44000,
                }],
            })
        ```

        Creating an External Model endpoint

        ```python
        import pulumi
        import pulumi_databricks as databricks

        gpt4o = databricks.ModelServing("gpt_4o",
            name="gpt-4o-mini",
            ai_gateway={
                "usage_tracking_config": {
                    "enabled": True,
                },
                "rate_limits": [{
                    "calls": 10,
                    "key": "endpoint",
                    "renewal_period": "minute",
                }],
                "inference_table_config": {
                    "enabled": True,
                    "table_name_prefix": "gpt-4o-mini",
                    "catalog_name": "ml",
                    "schema_name": "ai_gateway",
                },
                "guardrails": {
                    "input": {
                        "invalid_keywords": ["SuperSecretProject"],
                        "pii": {
                            "behavior": "BLOCK",
                        },
                    },
                    "output": {
                        "pii": {
                            "behavior": "BLOCK",
                        },
                    },
                },
            },
            config={
                "served_entities": [{
                    "name": "gpt-4o-mini",
                    "external_model": {
                        "name": "gpt-4o-mini",
                        "provider": "openai",
                        "task": "llm/v1/chat",
                        "openai_config": {
                            "openai_api_key": "{{secrets/llm_scope/openai_api_key}}",
                        },
                    },
                }],
            })
        ```

        ## Access Control

        * Permissions can control which groups or individual users can *Manage*, *Query* or *View* individual serving endpoints.

        ## Related Resources

        The following resources are often used in the same context:

        * ModelServingProvisionedThroughput to create [Foundation Model provisioned throughput](https://docs.databricks.com/aws/en/machine-learning/foundation-model-apis/deploy-prov-throughput-foundation-model-apis) endpoints in Databricks.
        * RegisteredModel to create [Models in Unity Catalog](https://docs.databricks.com/en/mlflow/models-in-uc.html) in Databricks.
        * End to end workspace management guide.
        * Directory to manage directories in [Databricks Workspace](https://docs.databricks.com/workspace/workspace-objects.html).
        * MlflowModel to create models in the [workspace model registry](https://docs.databricks.com/en/mlflow/model-registry.html) in Databricks.
        * Notebook to manage [Databricks Notebooks](https://docs.databricks.com/notebooks/index.html).
        * Notebook data to export a notebook from Databricks Workspace.
        * Repo to manage [Databricks Repos](https://docs.databricks.com/repos.html).

        ## Import

        The model serving resource can be imported using the name of the endpoint.

        hcl

        import {

          to = databricks_model_serving.this

          id = "<model-serving-endpoint-name>"

        }

        Alternatively, when using `terraform` version 1.4 or earlier, import using the `pulumi import` command:

        bash

        ```sh
        $ pulumi import databricks:index/modelServing:ModelServing this <model-serving-endpoint-name>
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Union['ModelServingAiGatewayArgs', 'ModelServingAiGatewayArgsDict']] ai_gateway: A block with AI Gateway configuration for the serving endpoint. *Note: only external model endpoints are supported as of now.*
        :param pulumi.Input[_builtins.str] budget_policy_id: The Budget Policy ID set for this serving endpoint.
        :param pulumi.Input[Union['ModelServingConfigArgs', 'ModelServingConfigArgsDict']] config: The model serving endpoint configuration. This is optional and can be added and modified after creation. If `config` was provided in a previous apply but is not provided in the current apply, no change to the model serving endpoint will occur. To recreate the model serving endpoint without the `config` block, the model serving endpoint must be destroyed and recreated.
        :param pulumi.Input[_builtins.str] description: The description of the model serving endpoint.
        :param pulumi.Input[Union['ModelServingEmailNotificationsArgs', 'ModelServingEmailNotificationsArgsDict']] email_notifications: A block with Email notification setting.
        :param pulumi.Input[_builtins.str] name: The name of the model serving endpoint. This field is required and must be unique across a workspace. An endpoint name can consist of alphanumeric characters, dashes, and underscores. NOTE: Changing this name will delete the existing endpoint and create a new endpoint with the updated name.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ModelServingRateLimitArgs', 'ModelServingRateLimitArgsDict']]]] rate_limits: A list of rate limit blocks to be applied to the serving endpoint. *Note: only external and foundation model endpoints are supported as of now.*
        :param pulumi.Input[_builtins.bool] route_optimized: A boolean enabling route optimization for the endpoint. *Note: only available for custom models.*
        :param pulumi.Input[Sequence[pulumi.Input[Union['ModelServingTagArgs', 'ModelServingTagArgsDict']]]] tags: Tags to be attached to the serving endpoint and automatically propagated to billing logs.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[ModelServingArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        This resource allows you to manage [Model Serving](https://docs.databricks.com/machine-learning/model-serving/index.html) endpoints in Databricks, including custom models, external models, and foundation models. For newer foundation models, including Llama 4, please use the ModelServingProvisionedThroughput resource.

        > This resource can only be used with a workspace-level provider!

        > If you replace `served_models` with `served_entities` in an existing serving endpoint, the serving endpoint will briefly go into an update state (~30 seconds) and increment the config version.

        ## Example Usage

        Creating a CPU serving endpoint

        ```python
        import pulumi
        import pulumi_databricks as databricks

        this = databricks.ModelServing("this",
            name="ads-serving-endpoint",
            config={
                "served_entities": [
                    {
                        "name": "prod_model",
                        "entity_name": "ads-model",
                        "entity_version": "2",
                        "workload_size": "Small",
                        "scale_to_zero_enabled": True,
                    },
                    {
                        "name": "candidate_model",
                        "entity_name": "ads-model",
                        "entity_version": "4",
                        "workload_size": "Small",
                        "scale_to_zero_enabled": False,
                    },
                ],
                "traffic_config": {
                    "routes": [
                        {
                            "served_model_name": "prod_model",
                            "traffic_percentage": 90,
                        },
                        {
                            "served_model_name": "candidate_model",
                            "traffic_percentage": 10,
                        },
                    ],
                },
            })
        ```

        Creating a Foundation Model endpoint

        ```python
        import pulumi
        import pulumi_databricks as databricks

        llama = databricks.ModelServing("llama",
            name="llama_3_2_3b_instruct",
            ai_gateway={
                "usage_tracking_config": {
                    "enabled": True,
                },
            },
            config={
                "served_entities": [{
                    "name": "meta_llama_v3_2_3b_instruct-3",
                    "entity_name": "system.ai.llama_v3_2_3b_instruct",
                    "entity_version": "2",
                    "scale_to_zero_enabled": True,
                    "max_provisioned_throughput": 44000,
                }],
            })
        ```

        Creating an External Model endpoint

        ```python
        import pulumi
        import pulumi_databricks as databricks

        gpt4o = databricks.ModelServing("gpt_4o",
            name="gpt-4o-mini",
            ai_gateway={
                "usage_tracking_config": {
                    "enabled": True,
                },
                "rate_limits": [{
                    "calls": 10,
                    "key": "endpoint",
                    "renewal_period": "minute",
                }],
                "inference_table_config": {
                    "enabled": True,
                    "table_name_prefix": "gpt-4o-mini",
                    "catalog_name": "ml",
                    "schema_name": "ai_gateway",
                },
                "guardrails": {
                    "input": {
                        "invalid_keywords": ["SuperSecretProject"],
                        "pii": {
                            "behavior": "BLOCK",
                        },
                    },
                    "output": {
                        "pii": {
                            "behavior": "BLOCK",
                        },
                    },
                },
            },
            config={
                "served_entities": [{
                    "name": "gpt-4o-mini",
                    "external_model": {
                        "name": "gpt-4o-mini",
                        "provider": "openai",
                        "task": "llm/v1/chat",
                        "openai_config": {
                            "openai_api_key": "{{secrets/llm_scope/openai_api_key}}",
                        },
                    },
                }],
            })
        ```

        ## Access Control

        * Permissions can control which groups or individual users can *Manage*, *Query* or *View* individual serving endpoints.

        ## Related Resources

        The following resources are often used in the same context:

        * ModelServingProvisionedThroughput to create [Foundation Model provisioned throughput](https://docs.databricks.com/aws/en/machine-learning/foundation-model-apis/deploy-prov-throughput-foundation-model-apis) endpoints in Databricks.
        * RegisteredModel to create [Models in Unity Catalog](https://docs.databricks.com/en/mlflow/models-in-uc.html) in Databricks.
        * End to end workspace management guide.
        * Directory to manage directories in [Databricks Workspace](https://docs.databricks.com/workspace/workspace-objects.html).
        * MlflowModel to create models in the [workspace model registry](https://docs.databricks.com/en/mlflow/model-registry.html) in Databricks.
        * Notebook to manage [Databricks Notebooks](https://docs.databricks.com/notebooks/index.html).
        * Notebook data to export a notebook from Databricks Workspace.
        * Repo to manage [Databricks Repos](https://docs.databricks.com/repos.html).

        ## Import

        The model serving resource can be imported using the name of the endpoint.

        hcl

        import {

          to = databricks_model_serving.this

          id = "<model-serving-endpoint-name>"

        }

        Alternatively, when using `terraform` version 1.4 or earlier, import using the `pulumi import` command:

        bash

        ```sh
        $ pulumi import databricks:index/modelServing:ModelServing this <model-serving-endpoint-name>
        ```

        :param str resource_name: The name of the resource.
        :param ModelServingArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ModelServingArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 ai_gateway: Optional[pulumi.Input[Union['ModelServingAiGatewayArgs', 'ModelServingAiGatewayArgsDict']]] = None,
                 budget_policy_id: Optional[pulumi.Input[_builtins.str]] = None,
                 config: Optional[pulumi.Input[Union['ModelServingConfigArgs', 'ModelServingConfigArgsDict']]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 email_notifications: Optional[pulumi.Input[Union['ModelServingEmailNotificationsArgs', 'ModelServingEmailNotificationsArgsDict']]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 rate_limits: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ModelServingRateLimitArgs', 'ModelServingRateLimitArgsDict']]]]] = None,
                 route_optimized: Optional[pulumi.Input[_builtins.bool]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ModelServingTagArgs', 'ModelServingTagArgsDict']]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ModelServingArgs.__new__(ModelServingArgs)

            __props__.__dict__["ai_gateway"] = ai_gateway
            __props__.__dict__["budget_policy_id"] = budget_policy_id
            __props__.__dict__["config"] = config
            __props__.__dict__["description"] = description
            __props__.__dict__["email_notifications"] = email_notifications
            __props__.__dict__["name"] = name
            __props__.__dict__["rate_limits"] = rate_limits
            __props__.__dict__["route_optimized"] = route_optimized
            __props__.__dict__["tags"] = tags
            __props__.__dict__["endpoint_url"] = None
            __props__.__dict__["serving_endpoint_id"] = None
        super(ModelServing, __self__).__init__(
            'databricks:index/modelServing:ModelServing',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            ai_gateway: Optional[pulumi.Input[Union['ModelServingAiGatewayArgs', 'ModelServingAiGatewayArgsDict']]] = None,
            budget_policy_id: Optional[pulumi.Input[_builtins.str]] = None,
            config: Optional[pulumi.Input[Union['ModelServingConfigArgs', 'ModelServingConfigArgsDict']]] = None,
            description: Optional[pulumi.Input[_builtins.str]] = None,
            email_notifications: Optional[pulumi.Input[Union['ModelServingEmailNotificationsArgs', 'ModelServingEmailNotificationsArgsDict']]] = None,
            endpoint_url: Optional[pulumi.Input[_builtins.str]] = None,
            name: Optional[pulumi.Input[_builtins.str]] = None,
            rate_limits: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ModelServingRateLimitArgs', 'ModelServingRateLimitArgsDict']]]]] = None,
            route_optimized: Optional[pulumi.Input[_builtins.bool]] = None,
            serving_endpoint_id: Optional[pulumi.Input[_builtins.str]] = None,
            tags: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ModelServingTagArgs', 'ModelServingTagArgsDict']]]]] = None) -> 'ModelServing':
        """
        Get an existing ModelServing resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Union['ModelServingAiGatewayArgs', 'ModelServingAiGatewayArgsDict']] ai_gateway: A block with AI Gateway configuration for the serving endpoint. *Note: only external model endpoints are supported as of now.*
        :param pulumi.Input[_builtins.str] budget_policy_id: The Budget Policy ID set for this serving endpoint.
        :param pulumi.Input[Union['ModelServingConfigArgs', 'ModelServingConfigArgsDict']] config: The model serving endpoint configuration. This is optional and can be added and modified after creation. If `config` was provided in a previous apply but is not provided in the current apply, no change to the model serving endpoint will occur. To recreate the model serving endpoint without the `config` block, the model serving endpoint must be destroyed and recreated.
        :param pulumi.Input[_builtins.str] description: The description of the model serving endpoint.
        :param pulumi.Input[Union['ModelServingEmailNotificationsArgs', 'ModelServingEmailNotificationsArgsDict']] email_notifications: A block with Email notification setting.
        :param pulumi.Input[_builtins.str] endpoint_url: Invocation url of the endpoint.
        :param pulumi.Input[_builtins.str] name: The name of the model serving endpoint. This field is required and must be unique across a workspace. An endpoint name can consist of alphanumeric characters, dashes, and underscores. NOTE: Changing this name will delete the existing endpoint and create a new endpoint with the updated name.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ModelServingRateLimitArgs', 'ModelServingRateLimitArgsDict']]]] rate_limits: A list of rate limit blocks to be applied to the serving endpoint. *Note: only external and foundation model endpoints are supported as of now.*
        :param pulumi.Input[_builtins.bool] route_optimized: A boolean enabling route optimization for the endpoint. *Note: only available for custom models.*
        :param pulumi.Input[_builtins.str] serving_endpoint_id: Unique identifier of the serving endpoint primarily used to set permissions and refer to this instance for other operations.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ModelServingTagArgs', 'ModelServingTagArgsDict']]]] tags: Tags to be attached to the serving endpoint and automatically propagated to billing logs.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ModelServingState.__new__(_ModelServingState)

        __props__.__dict__["ai_gateway"] = ai_gateway
        __props__.__dict__["budget_policy_id"] = budget_policy_id
        __props__.__dict__["config"] = config
        __props__.__dict__["description"] = description
        __props__.__dict__["email_notifications"] = email_notifications
        __props__.__dict__["endpoint_url"] = endpoint_url
        __props__.__dict__["name"] = name
        __props__.__dict__["rate_limits"] = rate_limits
        __props__.__dict__["route_optimized"] = route_optimized
        __props__.__dict__["serving_endpoint_id"] = serving_endpoint_id
        __props__.__dict__["tags"] = tags
        return ModelServing(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="aiGateway")
    def ai_gateway(self) -> pulumi.Output[Optional['outputs.ModelServingAiGateway']]:
        """
        A block with AI Gateway configuration for the serving endpoint. *Note: only external model endpoints are supported as of now.*
        """
        return pulumi.get(self, "ai_gateway")

    @_builtins.property
    @pulumi.getter(name="budgetPolicyId")
    def budget_policy_id(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The Budget Policy ID set for this serving endpoint.
        """
        return pulumi.get(self, "budget_policy_id")

    @_builtins.property
    @pulumi.getter
    def config(self) -> pulumi.Output['outputs.ModelServingConfig']:
        """
        The model serving endpoint configuration. This is optional and can be added and modified after creation. If `config` was provided in a previous apply but is not provided in the current apply, no change to the model serving endpoint will occur. To recreate the model serving endpoint without the `config` block, the model serving endpoint must be destroyed and recreated.
        """
        return pulumi.get(self, "config")

    @_builtins.property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The description of the model serving endpoint.
        """
        return pulumi.get(self, "description")

    @_builtins.property
    @pulumi.getter(name="emailNotifications")
    def email_notifications(self) -> pulumi.Output[Optional['outputs.ModelServingEmailNotifications']]:
        """
        A block with Email notification setting.
        """
        return pulumi.get(self, "email_notifications")

    @_builtins.property
    @pulumi.getter(name="endpointUrl")
    def endpoint_url(self) -> pulumi.Output[_builtins.str]:
        """
        Invocation url of the endpoint.
        """
        return pulumi.get(self, "endpoint_url")

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Output[_builtins.str]:
        """
        The name of the model serving endpoint. This field is required and must be unique across a workspace. An endpoint name can consist of alphanumeric characters, dashes, and underscores. NOTE: Changing this name will delete the existing endpoint and create a new endpoint with the updated name.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="rateLimits")
    @_utilities.deprecated("""Please use AI Gateway to manage rate limits.""")
    def rate_limits(self) -> pulumi.Output[Optional[Sequence['outputs.ModelServingRateLimit']]]:
        """
        A list of rate limit blocks to be applied to the serving endpoint. *Note: only external and foundation model endpoints are supported as of now.*
        """
        return pulumi.get(self, "rate_limits")

    @_builtins.property
    @pulumi.getter(name="routeOptimized")
    def route_optimized(self) -> pulumi.Output[Optional[_builtins.bool]]:
        """
        A boolean enabling route optimization for the endpoint. *Note: only available for custom models.*
        """
        return pulumi.get(self, "route_optimized")

    @_builtins.property
    @pulumi.getter(name="servingEndpointId")
    def serving_endpoint_id(self) -> pulumi.Output[_builtins.str]:
        """
        Unique identifier of the serving endpoint primarily used to set permissions and refer to this instance for other operations.
        """
        return pulumi.get(self, "serving_endpoint_id")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Sequence['outputs.ModelServingTag']]]:
        """
        Tags to be attached to the serving endpoint and automatically propagated to billing logs.
        """
        return pulumi.get(self, "tags")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetBudgetPolicyResult',
    'AwaitableGetBudgetPolicyResult',
    'get_budget_policy',
    'get_budget_policy_output',
]

@pulumi.output_type
class GetBudgetPolicyResult:
    """
    A collection of values returned by getBudgetPolicy.
    """
    def __init__(__self__, binding_workspace_ids=None, custom_tags=None, id=None, policy_id=None, policy_name=None):
        if binding_workspace_ids and not isinstance(binding_workspace_ids, list):
            raise TypeError("Expected argument 'binding_workspace_ids' to be a list")
        pulumi.set(__self__, "binding_workspace_ids", binding_workspace_ids)
        if custom_tags and not isinstance(custom_tags, list):
            raise TypeError("Expected argument 'custom_tags' to be a list")
        pulumi.set(__self__, "custom_tags", custom_tags)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if policy_id and not isinstance(policy_id, str):
            raise TypeError("Expected argument 'policy_id' to be a str")
        pulumi.set(__self__, "policy_id", policy_id)
        if policy_name and not isinstance(policy_name, str):
            raise TypeError("Expected argument 'policy_name' to be a str")
        pulumi.set(__self__, "policy_name", policy_name)

    @_builtins.property
    @pulumi.getter(name="bindingWorkspaceIds")
    def binding_workspace_ids(self) -> Sequence[_builtins.int]:
        """
        (list of integer) - List of workspaces that this budget policy will be exclusively bound to.
        An empty binding implies that this budget policy is open to any workspace in the account
        """
        return pulumi.get(self, "binding_workspace_ids")

    @_builtins.property
    @pulumi.getter(name="customTags")
    def custom_tags(self) -> Sequence['outputs.GetBudgetPolicyCustomTagResult']:
        """
        (list of CustomPolicyTag) - A list of tags defined by the customer. At most 20 entries are allowed per policy
        """
        return pulumi.get(self, "custom_tags")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="policyId")
    def policy_id(self) -> _builtins.str:
        """
        (string) - The Id of the policy. This field is generated by Databricks and globally unique
        """
        return pulumi.get(self, "policy_id")

    @_builtins.property
    @pulumi.getter(name="policyName")
    def policy_name(self) -> _builtins.str:
        """
        (string) - The name of the policy.
        - Must be unique among active policies.
        - Can contain only characters from the ISO 8859-1 (latin1) set.
        - Can't start with reserved keywords such as `databricks:default-policy`
        """
        return pulumi.get(self, "policy_name")


class AwaitableGetBudgetPolicyResult(GetBudgetPolicyResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetBudgetPolicyResult(
            binding_workspace_ids=self.binding_workspace_ids,
            custom_tags=self.custom_tags,
            id=self.id,
            policy_id=self.policy_id,
            policy_name=self.policy_name)


def get_budget_policy(policy_id: Optional[_builtins.str] = None,
                      opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetBudgetPolicyResult:
    """
    [![Public Preview](https://img.shields.io/badge/Release_Stage-Public_Preview-yellowgreen)](https://docs.databricks.com/aws/en/release-notes/release-types)

    This data source can be used to get a single budget policy.

    > **Note** This data source can only be used with an account-level provider!

    ## Example Usage

    Referring to a budget policy by id:

    ```python
    import pulumi
    import pulumi_databricks as databricks

    this = databricks.get_budget_policy(policy_id="test")
    ```


    :param _builtins.str policy_id: The Id of the policy. This field is generated by Databricks and globally unique
    """
    __args__ = dict()
    __args__['policyId'] = policy_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('databricks:index/getBudgetPolicy:getBudgetPolicy', __args__, opts=opts, typ=GetBudgetPolicyResult).value

    return AwaitableGetBudgetPolicyResult(
        binding_workspace_ids=pulumi.get(__ret__, 'binding_workspace_ids'),
        custom_tags=pulumi.get(__ret__, 'custom_tags'),
        id=pulumi.get(__ret__, 'id'),
        policy_id=pulumi.get(__ret__, 'policy_id'),
        policy_name=pulumi.get(__ret__, 'policy_name'))
def get_budget_policy_output(policy_id: Optional[pulumi.Input[_builtins.str]] = None,
                             opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetBudgetPolicyResult]:
    """
    [![Public Preview](https://img.shields.io/badge/Release_Stage-Public_Preview-yellowgreen)](https://docs.databricks.com/aws/en/release-notes/release-types)

    This data source can be used to get a single budget policy.

    > **Note** This data source can only be used with an account-level provider!

    ## Example Usage

    Referring to a budget policy by id:

    ```python
    import pulumi
    import pulumi_databricks as databricks

    this = databricks.get_budget_policy(policy_id="test")
    ```


    :param _builtins.str policy_id: The Id of the policy. This field is generated by Databricks and globally unique
    """
    __args__ = dict()
    __args__['policyId'] = policy_id
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('databricks:index/getBudgetPolicy:getBudgetPolicy', __args__, opts=opts, typ=GetBudgetPolicyResult)
    return __ret__.apply(lambda __response__: GetBudgetPolicyResult(
        binding_workspace_ids=pulumi.get(__response__, 'binding_workspace_ids'),
        custom_tags=pulumi.get(__response__, 'custom_tags'),
        id=pulumi.get(__response__, 'id'),
        policy_id=pulumi.get(__response__, 'policy_id'),
        policy_name=pulumi.get(__response__, 'policy_name')))
